<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8">
    <!-- <link rel="icon" href="/favicon.ico"> -->
    <link rel="icon" href="{{ url_for('static', filename='/favicon.ico') }}" type="image/x-icon">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Si Home</title>
    <script type="module" crossorigin src="/assets/index-75db7cea.js"></script>
    <link rel="stylesheet" href="/assets/index-5f8a0728.css">
  </head>
  <body>
    <div id="app"></div>
    
  </body>
</html>
